/*--------------------------------------------------------------------
 * i2ccom.c - IC Low-Level Communication Layer
 *--------------------------------------------------------------------
 * $Id: i2ccom.c,v 1.1.1.1 2009-09-02 08:45:11 jtharin Exp $
 *--------------------------------------------------------------------
 * $Author: jtharin $
 * $Date: 2009-09-02 08:45:11 $
 * $Revision: 1.5 $
 *--------------------------------------------------------------------
 * $Log: i2ccom.c,v $
 * Revision 1.5  2009-09-02 08:45:11  jtharin
 * Imported using TkCVS
 *
 * Revision 1.1.1.1  2009/08/28 12:19:18  root
 * Imported using TkCVS
 *
 * Revision 1.4  2006/03/07 17:39:57  pbureau
 * Makefile update for clean x86 compiling
 * i2c compilation cleaned for 2.6 kernel
 *
 * Revision 1.3  2004/09/22 18:55:08  pbureau
 * *** empty log message ***
 *
 * Revision 1.1  2004/07/29 10:51:54  cgaudin
 * New libkorebot release 1.2
 *
 *--------------------------------------------------------------------*/

#include "i2ccom.h"

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <errno.h>

#include <stdio.h>

//#include <stdio.h>

//#include <linux/i2c.h>
//#include <linux/i2c-dev.h>
#include "i2c-dev.h"

/*! 
 * \file   i2ccom.c IC Low-Level Communication Layer             
 *
 * \brief 
 *         This module is the low-level communication layer between   
 *         the KoreBot Library and the Linux Operating System
 *        
 * \author   Yves Piguet (Calerga Sarl)                               
 * \author   Pierre Bureau (K-Team SA)                                
 * \author   Cdric Gaudin (K-Team SA)                                
 *
 * \note     Copyright (C) 2004 K-TEAM SA
 * \bug      none discovered.                                         


 * \todo     implement blocks bigger than 32 bytes

 */

/*--------------------------------------------------------------------*/
/*! 
 * This function opens an IC Bus. 
 *
 * \param i2c    Pointer to IC Bus Descriptor. 
 *               This descriptor must be allocated by the caller.
 * \param device A string representing the device name. 
 *               A NULL should be used for default IC Bus. 
 *               (e.g. /dev/i2c/0)
 * \return an error code
 *          - <0 standard error number. See errno.h
 *          - 0 if IC Bus Device has been opened
 */   
int i2c_open( i2c_t *i2c , char const *device )
{
  /* no device selected */
  i2c->dev = -1;      
    
  /* set default IC bus device */
  if (!device)
    device = "/dev/i2c-0";

  if ((i2c->fd = open( device , O_RDWR )) < 0 )
    return i2c->fd;
  
  return 0;
}

/*--------------------------------------------------------------------*/
/*! 
 * This procedure closes an IC Bus. 
 * 
 * \param i2c  IC Bus Handle
 */
void i2c_close( i2c_t *i2c )
{
  if (i2c->fd>0) { 
    close(i2c->fd);
    i2c->fd = -1;
  }
}

/*---------------------------------------------------------------------*/
/*! 
 * This function selects an IC Device on a given IC Bus.
 *
 * \param i2c     IC Bus Handle
 * \param address IC Device Address. This value contains only 
 *                the IC address bits (7-bit or 10-bit).   
 * \return an error code
 *         - <0 standard error code. See errno.h 
 *         - 0 on success
 *
 * \remarks This function is used internally and is never 
 *          exported outside this module. 
 */
static int i2c_priv_select_device( i2c_t * i2c , i2c_dev_t address )
{
  int rc;

  /* change device only if necessary */
  if (i2c->dev != address ) {

    if ((rc=ioctl( i2c->fd , I2C_SLAVE , address )) < 0)
      return rc;

    i2c->dev = address;
  }
  return 0;
}

/*--------------------------------------------------------------------*/
/*! 
 * This function is the primitive to read data from a given device 
 * on a given IC Bus.
 *
 * \param i2c IC Bus Handle
 * \param dev IC Device Address. This value contains only 
 *            the IC address bits (7-bit or 10-bit).   
 * \param buf Pointer to the buffer that will receive the data bytes.
 * \param len Size of the buffer in bytes.
 * \return an error code:
 *         - <0 standard error number. See errno.h
 *         - >=0 number of bytes read
 */



int i2c_llread( i2c_t * i2c , 
		i2c_dev_t dev , 
		char * buf , 
		unsigned int len )
{
  int rc,i;
  int reg=0;
  
  char buf0[I2C_SMBUS_I2C_BLOCK_MAX];
  
  i2c_priv_select_device( i2c , dev );

  //rc = read( i2c->fd , buf , len );
  reg = buf[0]; // copy register
  
  //buf0[0]=len;   // save length to be read
  
  
  if (len <= I2C_SMBUS_I2C_BLOCK_MAX) {
  
  
   if (len == 1)
   {
    buf0[0] =  i2c_smbus_read_byte_data(i2c->fd,reg); 
    
    rc=1; // to test errno
    
    
    }
   else
   {
    rc = i2c_smbus_read_i2c_block_data(i2c->fd,reg,len,(__u8 *) buf0);
   }
    
    
    
    
    // copy read values
    if (rc>=0) 
    {
    
      rc=len;  // seems to be an error in i2c_smbus_read_block_data: the read length data is always 32..
      for (i=0;i<rc;i++)
        {
          buf[i]=buf0[i];
        }
    }
  
  } else
  {
     // if data to read is bigger than I2C_SMBUS_I2C_BLOCK_MAX, should implement a loop
    rc = -1;
  }
  
  return rc;
}

/*--------------------------------------------------------------------*/
/*! 
 * This function is the primitive to write data to a given device 
 * on a given IC Bus.
 * 
 * \param i2c IC Bus Handle
 * \param dev IC Device Address. This value contains only 
 *            the IC address bits (7-bit or 10-bit).  
 * \param buf Pointer to the buffer that will be written.
 * \param len Number of bytes to write
 * \return an error code:
 *         - <0 standard error number. See errno.h 
 *         - >=0 number of bytes written 
 */
int i2c_llwrite( i2c_t * i2c , 
		 i2c_dev_t dev , 
		 const char * buf , 
		 unsigned int len )
{
  int rc;
  int reg=0;
  
  i2c_priv_select_device( i2c , dev );
  
  // rc = write( i2c->fd , buf , len );
  
  reg = buf[0];
  len=len-1;
  
  if (len == 1)
  {
    rc=i2c_smbus_write_byte_data(i2c->fd,reg,(__u8)buf[1]);
      
  }
  else {
   if (len <= I2C_SMBUS_I2C_BLOCK_MAX) {
    rc = i2c_smbus_write_i2c_block_data(i2c->fd,reg,len,(__u8 *)buf+1);
    
    } else
    {
       // if data to write is bigger than I2C_SMBUS_I2C_BLOCK_MAX, should implement a loop
      rc=-1;
    }
  }
  

  return rc;
}


/*--------------------------------------------------------------------*/
/*! 
 *
 * This function is the primitive to write data to and then read 
 * data from a given device on a given IC Bus.
 * 
 * \param i2c       IC Bus Handle.
 * \param dev       IC Device Address. This value contains only 
 *                  the IC address bits (7-bit or 10-bit).   
 * \param write_buf Pointer to the buffer that contains the data 
 *                  to b e written. 
 * \param write_len Number of bytes to write. The data are in 
 *                  the write buffer.  
 * \param read_buf  Pointer to the buffer that will receive the 
 *                  data. 
 * \param read_len  Size of the read buffer in bytes.
 * \return an error code:
 *         - <0  standard error code. See errno.h
 *         - >=0 on success 
 */
int i2c_lltransfer( i2c_t * i2c , i2c_dev_t dev , 
		    const char * write_buf , unsigned int write_len ,
		    char * read_buf , unsigned int read_len )
{
  struct i2c_msg             msgs[2];
  struct i2c_rdwr_ioctl_data msgset;
  int rc,i;  
	int reg=0;

  char buf0[I2C_SMBUS_I2C_BLOCK_MAX];


/* I2C_RDWR is not working for SMB bus
 //  Write Message
  msgs[0].addr  = dev;
  msgs[0].flags = 0;
  msgs[0].buf   = (char *)write_buf;
  msgs[0].len   = write_len;
      
  // Read Message 
  msgs[1].addr  = dev;
  msgs[1].flags = I2C_M_RD;
  msgs[1].buf   = read_buf;
  msgs[1].len   = read_len;
  	
  msgset.msgs  = msgs;
  msgset.nmsgs = 2;
	
  rc = ioctl( i2c->fd , I2C_RDWR , &msgset ); 
 
 */
  
  // write 
  reg = write_buf[0]; // register
  
  /*if (write_len >1) {
  
  fprintf (stderr,"In i2ccom,i2c_lltransfer function: nb of bytes to write too big: %d\n",write_len);
  
  return -1;
  
  }*/
  
  /*if (write_len == 1)
  {
    rc=i2c_smbus_write_byte_data(i2c->fd,reg,(__u8 )write_buf[1]);
    

  }
  else {
  
    rc = i2c_smbus_write_i2c_block_data(i2c->fd,reg,write_len,(__u8 *)write_buf+1);
  }*/
  
  rc=0;
  
  //rc = i2c_smbus_write_byte_data(i2c->fd,reg,write_buf[0]);
  if (rc >= 0) {
    // read
    
    //read_buf[0]=read_len;
      buf0[0]=read_len;   // save length to be read
   /* if (read_len >1) {
    
    fprintf (stderr,"In i2ccom,i2c_lltransfer function: nb of bytes to read too big: %d\n",read_len);
    
    return -1;
    
    }*/
    

    //rc = i2c_smbus_read_byte_data(i2c->fd,reg);
    
      
    
    //read_buf[0]=rc;
    
    
    // read
    
    if (rc>=0)
    {
      if (read_len < I2C_SMBUS_I2C_BLOCK_MAX) {

      
         if (read_len == 1)
         {
             
           buf0[0] =  i2c_smbus_read_byte_data(i2c->fd,reg); 
           rc=1; // to test errno
          }
         else
         {
           rc = i2c_smbus_read_i2c_block_data(i2c->fd,reg,read_len,(__u8 *)buf0); // read data 
         }
      
      
        // copy read values
        if (rc>=0) 
        {
        
         // rc=read_len;  // seems to be an error in i2c_smbus_read_block_data: the read length data is always 32..
          for (i=0;i<read_len;i++)
            {
              read_buf[i]=buf0[i];
            }
        }
    
    } else
    {
      // if data to read is bigger than I2C_SMBUS_I2C_BLOCK_MAX, should implement a loop
      rc=-1;
    }
  }
 
 
 
  }
 else {
 	//printf("  Warning - write smbus (register) failed: res = %d\n",res);
//	perror( "  smbus write register failed" );
 // printf("\n\n");
 }
  
 
  return rc;
}

/*--------------------------------------------------------------------*/
/*! 
 * This function verifies the existance of a given device on 
 * a given IC Bus.
 * 
 * \param i2c  IC Bus Handle.
 * \param dev  IC Device Address. This value contains only 
 *             the IC address bits (7-bit or 10-bit).   
 * 
 * \return A value indicating if a device is present or not:
 *         - 1 device present 
 *         - 0 no device present
 */
int i2c_exists( i2c_t * i2c , i2c_dev_t dev )
{
  int rc, found=0;
 
 
 /* not valid  if adapter has not I2C_FUNC_I2C bit
 struct i2c_msg msg;
  struct i2c_rdwr_ioctl_data msgset;
 
  msg.addr  = dev;
  msg.flags = 0;
  msg.buf   = 0;
  msg.len   = 0;
    
  msgset.msgs  = &msg;
  msgset.nmsgs = 1;

   rc = ioctl( i2c->fd , I2C_RDWR , &msgset );
 */ 
 


//  printf("begin i2c_exists\n");



  rc=ioctl( i2c->fd , I2C_SLAVE ,dev ); // change slave address
    
  //rc = i2c_smbus_read_byte(i2c->fd);
  rc = i2c_smbus_write_quick(i2c->fd,0);
  //printf("i2ccom: slave 0x%0x  : rc %d\n",dev,rc);
  
  /* answer from an IC device */ 
  if ( rc >= 0 ) return 1;
    
  return 0;
} 
	   
/*--------------------------------------------------------------------*/
/*! 
 * This function scans a given IC Bus to find devices
 * 
 * \param i2c       IC Bus Handle.
 * \param callback  A callback function called when a device is found.
 *                  If this callback function returns a value <0 the 
 *                  bus scan stops immedialety and the value is 
 *                  used as return value by i2c_scan.
 * \param context   A value or a pointer passed to the callback 
 *                  function.
 * \return an error code:
 *         - <0 standard error code (See errno.h) or a user error code.
 *         - >=0 Number of devices found on the IC bus.
 */
int i2c_scan( i2c_t * i2c , 
	      int (*callback)( i2c_t * i2c , 
			       i2c_dev_t dev , 
			       void * context ) , 
	      void * context)
{
  int rc, adr, found=0;
  struct i2c_msg msg;
  struct i2c_rdwr_ioctl_data msgset;
 
  /* scan the whole IC bus range */
  for (adr=2; adr<128; adr++) {
    
    
    //  rc = ioctl( i2c->fd , I2C_RDWR , &msgset ); not valid  if adapter has not I2C_FUNC_I2C bit.
    /*msg.addr  = adr;
    msg.flags = 0;
    msg.buf   = 0;
    msg.len   = 0;
    
    msgset.msgs  = &msg;
    msgset.nmsgs = 1;
    
    rc = ioctl( i2c->fd , I2C_RDWR , &msgset );
    */
    
    //rc = i2c_smbus_read_byte_data(i2c->fd,adr);
    // rc = i2c_smbus_read_byte(i2c->fd);
    rc=ioctl( i2c->fd , I2C_SLAVE ,adr ); // change slave address
    
    //rc = i2c_smbus_read_byte(i2c->fd);
    rc = i2c_smbus_write_quick(i2c->fd,0);
    
   // printf("Ad dress  : 0x%x | return code: %d\r\n",adr,rc);
   
    /* answer from a IC device */ 
    if ( rc >= 0 ) {
      found++;
      if ( callback != NULL ) {
	if ((rc=callback( i2c , adr , context ))<0)
	  return rc;
      }
    }
  }

  return found;
}

